// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "display/shader/SimpleTexturedShader.hpp"

using namespace Dawn;

void SimpleTexturedShader::getStages(
  const enum ShaderOpenGLVariant variant,
  const struct SimpleTexturedShaderData *rel,
  std::vector<std::shared_ptr<ShaderStage>> &stages,
  std::vector<struct ShaderOpenGLParameter> &parameters
) {
  // Stages
  std::shared_ptr<ShaderStage> vertex;
  std::shared_ptr<ShaderStage> fragment;

  switch(variant) {
    case ShaderOpenGLVariant::GLSL_330_CORE:
      vertex = std::make_shared<ShaderStage>(
        ShaderStageType::VERTEX,
        "#version 330 core\n"
        "layout (location = 0) in vec3 aPos;\n"
        "uniform mat4 u_Projection;\n"
        "uniform mat4 u_View;\n"
        "uniform mat4 u_Model;\n"
        "void main() {\n"
          "gl_Position = u_Projection * u_View * u_Model * vec4(aPos, 1.0);\n"
        "}"
      );
      
      fragment = std::make_shared<ShaderStage>(
        ShaderStageType::FRAGMENT,
        "#version 330 core\n"
        "in vec2 o_TextCoord;\n"
        "out vec4 o_Color;\n"
        "uniform vec4 u_Color;\n"
        "void main() {\n"
          "o_Color = u_Color;"
        "}\n"
      );
      break;

    default:
      assertUnreachable("Unsupported ShaderOpenGLVariant");
  }

  // Add stages
  stages.push_back(vertex);
  stages.push_back(fragment);

  // Parameters
  parameters.push_back(ShaderOpenGLParameter(
    "u_Projection",
    &rel->projection,
    ShaderParameterType::MAT4
  ));

  parameters.push_back(ShaderOpenGLParameter(
    "u_View",
    &rel->view,
    ShaderParameterType::MAT4
  ));

  parameters.push_back(ShaderOpenGLParameter(
    "u_Model",
    &rel->model,
    ShaderParameterType::MAT4
  ));

  parameters.push_back(ShaderOpenGLParameter(
    "u_Color",
    &rel->color,
    ShaderParameterType::COLOR
  ));
}